﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
//***************************************************************
//Declare additional system-defined namespaces used by UDPClient
//***************************************************************
using System.Net.Sockets;
using System.Net;

namespace UDPListener
{
    public partial class Form1 : Form
    {
        //******************************************************
        //define variables that will be used throughout the code
        //******************************************************
        public UdpClient udpClient = new UdpClient(2424);
        public byte[] sendBytes;
        public byte[] data = new byte[1024];
        string PCIPAddress;
        public string strHostName;
        string ReturnIPAddress;
        List<IPEndPoint> IPList = new List<IPEndPoint>();
        public int DeviceNo, list;
        public IPEndPoint EtherIP;
        //********************************************************
        //the thread that will manage the data back from the board
        //********************************************************
        private System.Threading.Thread thdUDPReciever;

        //********************************************************
        //this Subroutine is to handle all the UDP return data
        //recieved from the thread UDPReciever
        //********************************************************
        public delegate void ReturnUDPDataCallback(byte[] text, IPEndPoint IP);

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //***********************************************
            //Declare and start UDP recieve Thread
            //***********************************************
            thdUDPReciever = new System.Threading.Thread(new System.Threading.ThreadStart(RecieveThread));
            thdUDPReciever.Start();

            //***********************************************
            //Find out PC IP address as UDP Recieve thread
            //recieves all UDP packets
            //***********************************************
            strHostName = System.Net.Dns.GetHostName();
            PCIPAddress = System.Net.Dns.GetHostEntry(strHostName).AddressList[0].ToString();
        }

        public void RecieveThread()
        {
            while (true)
            {
                //***********************************************
                //Declare RemoteIPEndPoint 
                //where RemotIPEndPoint is the any IP address 
                //and Port Number of the incoming UDP packet
                //***********************************************
                System.Net.IPEndPoint RemoteIpEndPoint = new System.Net.IPEndPoint(System.Net.IPAddress.Any, 0);
                //***********************************************
                //In remoteIPendpoint the IP address can be changed from any to a specific
                //IP address and the Port which is 0 in the above line can be changed 
                //to a specific port number and it will only recieve data from that port number.
                //***********************************************

                //***********************************************
                //Declare other variables used
                //***********************************************
                byte[] receiveBytes;
                string returnData;

                //***********************************************
                //Receive incoming UDP Packet
                //***********************************************
                receiveBytes = udpClient.Receive(ref RemoteIpEndPoint);

                returnData = System.Text.Encoding.ASCII.GetString(receiveBytes);

                //***********************************************
                // test to see if there is anything to recieve
                //***********************************************
                if (returnData.Length != 0) 
                {
                    //***********************************************
                    //Filter data by IP address 
                    //if from PC IP address ignore 
                    //else accept incoming data
                    //***********************************************
                    if ((PCIPAddress == RemoteIpEndPoint.Address.ToString()))
                    {
                        //ignore any data sent by PC running application
                    }
                    else
                    {
                        if (ReturnIPAddress != RemoteIpEndPoint.Address.ToString())
                        {
                            //*********************************************
                            // Setup list of IP address that have responded
                            //*********************************************
                            IPList.Insert(list, RemoteIpEndPoint);
                            list = list + 1;
                        }

                        ReturnIPAddress = RemoteIpEndPoint.Address.ToString();

                        //***********************************************
                        //Return Data that was recieved from Ether I/O as well as IP address
                        //***********************************************
                        ReturnUDPData(receiveBytes, RemoteIpEndPoint);
                        
                    }
                }
            }
        }

        public void ReturnUDPData(byte[] UDPData, IPEndPoint RemoteIP)
        {
            //**************************
            //Declare Variables
            //**************************
            string MacString;
            string VersionNumber;
           
            //*******************************************************
            //if 12 bytes were recieved in UDP Packet from the device 
            //then we have recived IO24 + MAC +Version
            //*******************************************************

            if ((UDPData.Length == 12))
            {
                
                if (this.comboBox1.InvokeRequired)
                {
                    //*************************************************
                    //Need to invoke as the combobox and UDP reciever 
                    //is operating on different threads
                    //*************************************************
                    ReturnUDPDataCallback d = new ReturnUDPDataCallback(ReturnUDPData);
                    this.comboBox1.Invoke(d, new object[] { UDPData, RemoteIP });
                }
                else
                {
                    //*************************************************
                    //Build up MAC string from the UDP data recieved
                    //*************************************************
                    MacString = "  MAC:" + Convert.ToString(UDPData[4], 16) + ":";
                    MacString = MacString + Convert.ToString(UDPData[5], 16) + ":";
                    MacString = MacString + Convert.ToString(UDPData[6], 16) + ":";
                    MacString = MacString + Convert.ToString(UDPData[7], 16) + ":";
                    MacString = MacString + Convert.ToString(UDPData[8], 16) + ":";
                    MacString = MacString + Convert.ToString(UDPData[9], 16);
                    MacString = MacString.ToUpper();
                    
                    //*************************************************
                    //Build up Version Number string from the UDP data recieved
                    //*************************************************
                    VersionNumber = Convert.ToString(UDPData[10], 16) + ".";
                    VersionNumber = VersionNumber + Convert.ToString(UDPData[11], 16);

                    //*************************************************
                    //Add Ether IP address & MAC Address to combo box
                    //*************************************************
                    
                    this.comboBox1.Items.Add(RemoteIP.Address.ToString() + MacString + "   V" + VersionNumber);
                    this.comboBox1.SelectedIndex = comboBox1.Items.Count - 1;

                    DeviceNo = comboBox1.SelectedIndex;
                    EtherIP = IPList[DeviceNo];

                }

            }
            if ((UDPData.Length == 3))
            {
                if (UDPData[0] == 0x21 && UDPData[1] == 0x41) //testing !A
                {
                    if (this.textBox1.InvokeRequired)
                    {
                        ReturnUDPDataCallback d = new ReturnUDPDataCallback(ReturnUDPData);
                        this.textBox1.Invoke(d, new object[] { UDPData, RemoteIP });
                    }
                    else
                    {
                        this.textBox1.Text = UDPData[0] + UDPData[1] + Convert.ToString(UDPData[2], 16);
                        ////read value
                        
                    }

                }
                
            }


        }
        private void Ether_Scan_Click(object sender, EventArgs e)
        {
            //******************************************************
            //Send out IO24 via broadcast(255.255.255.255) on port 2424
            //******************************************************
            //******************************************************
            //fill sendBytes buffer with "IO24"
            //******************************************************
            sendBytes = Encoding.ASCII.GetBytes("IO24");

            //******************************************************
            //Broadcast UDP packet with IO24 on port 2424 
            //using the udpClient.send 
            //******************************************************
            udpClient.Send(sendBytes,           //buffer
                            sendBytes.Length,   //buffer length
                            "255.255.255.255",  //destination IP
                            2424);              //destination Port
            //******************************************************
            //All units will respond with IO24 
            //+ 6 byte MAC address + Version Number
            //******************************************************

        }

        private void Read_PORTA_Click(object sender, EventArgs e)
        {
            //Command data
            data[0] = Convert.ToByte('a');//"a"
            //Sending out Command Data
            udpClient.Send(data, 1, EtherIP);
            //we expect a response of 2 bytes back from unit
            // "A" + port value of port A
            //this data will be captured 
            //and then dealt with in ReturnUDPData 
        }

        private void Read_DirA_Click(object sender, EventArgs e)
        {
            //Command data
            data[0] = Convert.ToByte('!');//"!"
            data[1] = Convert.ToByte('a');//"a"
            //Sending out Command Data
            udpClient.Send(data, 2, EtherIP);
            //we expect a response of 3 bytes back from unit
            //"!" + "A" + direction value of port A
            //this data will be captured 
            //and then dealt with in ReturnUDPData 
        }

        private void PortADirection_Click(object sender, EventArgs e)
        {
            //******************************************************
            //Declare buffer variable
            //******************************************************
            byte[] buffer = new byte[5];

            //******************************************************
            //Assign buffer values for command '!' + 'A' + port value
            //******************************************************
            buffer[0] = Convert.ToByte('!'); //"!"
            buffer[1] = Convert.ToByte('A');//"A"
            buffer[2] = 0x64; //"0x48"

            //******************************************************
            //Send out Command (!A 0x64) to 10.10.10.10 on port 2424
            //******************************************************
            udpClient.Send(buffer,          //buffer
                            3,              //buffer length
                            EtherIP);          //Destination Port

        }

        private void PortAValue_Click(object sender, EventArgs e)
        {
            //******************************************************
            //Declare buffer variable
            //******************************************************
            byte[] buffer = new byte[5];

            //******************************************************
            //Assign buffer values for command 'A' + port value
            //******************************************************
            buffer[0] = Convert.ToByte('A');//"A"
            buffer[1] = 0xFF; //"0xFF"

            //******************************************************
            //Send out Command (A 0xFF) to 10.10.10.10 on port 2424
            //******************************************************
            udpClient.Send(buffer,          //buffer
                            2,              //buffer length
                            EtherIP);  //Destination IP/PORT
                            

        }

        private void button1_Click(object sender, EventArgs e)
        {
            //Command data
            data[0] = Convert.ToByte('a');//"a"
            data[1] = Convert.ToByte('b');//"a"
            data[2] = Convert.ToByte('c');//"a"
            //Sending out Command Data
            udpClient.Send(data, 3, "255.255.255.255" , 2424);
            //we expect a response of 2 bytes back from unit
            // "A" + port value of port A
            //this data will be captured 
            //and then dealt with in ReturnUDPData 
        }
    }
}
